@@ -40,7 +40,7 @@ extern GladeXML *glade;
 extern GtkWidget *main_window;
 
 static GtkWidget *audio_profile;
-static GtkWidget *cd_option, *path_option, *file_option, *basepath_fcb, *check_strip, *check_eject;
+static GtkWidget *cd_option, *path_option, *file_option, *basepath_fcb, *check_strip, *check_eject, *check_open;
 static GtkWidget *path_example_label;
 
 #define DEFAULT_AUDIO_PROFILE_NAME "cdlossy"
@@ -215,13 +215,14 @@ static void pattern_label_update (void)
     "Beatles, The", /* sortname */
     NULL, /* genre */
     0, /* number of tracks*/
+    1, /* disc number */
     NULL, /* track list */
     NULL, /* release date */
     NULL, /* album ID */
     NULL /* artist ID */
   };
   static const TrackDetails sample_track = {
-    &sample_album,  /*album */
+    (AlbumDetails*)&sample_album,  /*album */
     7, /* track number */
     "Ticket To Ride", /* title */
     "The Beatles", /* artist */
@@ -372,7 +373,7 @@ void on_edit_preferences_cb (GtkMenuItem *item, gpointer user_data)
 
     prefs_dialog = glade_xml_get_widget (glade, "prefs_dialog");
     g_assert (prefs_dialog != NULL);
-    g_object_add_weak_pointer (G_OBJECT (prefs_dialog), (gpointer*)&prefs_dialog);
+    g_object_add_weak_pointer (G_OBJECT (prefs_dialog), (gpointer)&prefs_dialog);
 
     gtk_window_set_transient_for (GTK_WINDOW (prefs_dialog), GTK_WINDOW (main_window));
 
@@ -396,6 +397,7 @@ void on_edit_preferences_cb (GtkMenuItem *item, gpointer user_data)
     audio_profile = glade_xml_get_widget (glade, "audio_profile");
     check_strip = glade_xml_get_widget (glade, "check_strip");
     check_eject = glade_xml_get_widget (glade, "check_eject");
+    check_open = glade_xml_get_widget (glade, "check_open");
     path_example_label = glade_xml_get_widget (glade, "path_example_label");
 
     populate_pattern_combo (GTK_COMBO_BOX (path_option), path_patterns);
@@ -405,6 +407,7 @@ void on_edit_preferences_cb (GtkMenuItem *item, gpointer user_data)
 
     /* Connect to GConf to update the UI */
     gconf_bridge_bind_property (bridge, GCONF_EJECT, G_OBJECT (check_eject), "active");
+    gconf_bridge_bind_property (bridge, GCONF_OPEN, G_OBJECT (check_open), "active");
     gconf_bridge_bind_property (bridge, GCONF_STRIP, G_OBJECT (check_strip), "active");
     gconf_bridge_bind_property (bridge, GCONF_DEVICE, G_OBJECT (cd_option), "device");
     gconf_client_notify_add (gconf_client, GCONF_BASEURI, baseuri_changed_cb, NULL, NULL, NULL);
